const { logger } = require('./logger')
const jet = require('node-jet')
const sem = require('semaphore')(1)

exports.createDataBroker = function (peer) {
  return {
    startTransaction (callback) {
      sem.take(() => {
        // eslint-disable-next-line
        return callback({
          // CRUD Operations
          get (query) {
            return peer.get(query).catch(e => {
              logger.error('Failed to get from DB. Query: ', query)
              logger.error(e)
            })
          },
          set (resourceType, { id, value }) {
            return peer.set(`/${resourceType}/${id}`, value).catch(e => {
              logger.error('Failed to set in DB params:', {
                resourceType,
                id,
                value
              })
              logger.error(e)
            })
          },
          add (resourceType, { id, value }) {
            const resource = new jet.State(`/${resourceType}/${id}`, value)
            resource.on('set', newValue => newValue)
            return peer.add(resource)
          },
          remove (resourceType, id) {
            const resource = new jet.State(`/${resourceType}/${id}`)
            return peer.remove(resource)
          }
        }).then(() => sem.leave())
      })
    }
  }
}
